@@ -290,17 +290,14 @@ sub _set_gopath {
 }
 
 sub _set_gocache {
-    # Honor the user’s wishes if GOCACHE was explicitly set, e.g. for speeding
-    # up builds/tests during local package development.
     return if defined($ENV{GOCACHE}) && $ENV{GOCACHE} ne '';
 
-    # Explicitly setting the cache to off suppresses an error message when
-    # building with sbuild, where the default cache location is not writeable.
-    $ENV{GOCACHE} = "off";
+    # Set GOCACHE to a directory where the go command can safely write to.
+    my $this = shift;
+    $ENV{GOCACHE} = $this->{cwd} . '/' . $this->get_builddir() . '/go-build'
 }
 
 sub _set_go111module {
-    # Honor the user’s wishes if GO111MODULE was explicitly set.
     return if defined($ENV{GO111MODULE}) && $ENV{GO111MODULE} ne '';
 
     # Operate in "GOPATH mode" by default for "minimal module compatibility",
@@ -309,6 +306,13 @@ sub _set_go111module {
     $ENV{GO111MODULE} = "off";
 }
 
+sub _set_goproxy {
+    return if defined($ENV{GOPROXY}) && $ENV{GOPROXY} ne '';
+
+    # Disallow network access.
+    $ENV{GOPROXY} = "off";
+}
+
 sub _link_contents {
     my ($src, $dst) = @_;
 
@@ -475,6 +479,7 @@ sub build {
     $this->_set_gopath();
     $this->_set_gocache();
     $this->_set_go111module();
+    $this->_set_goproxy();
     if (exists($ENV{DH_GOLANG_GO_GENERATE}) && $ENV{DH_GOLANG_GO_GENERATE} == 1) {
         $this->doit_in_builddir("go", "generate", "-v", @_, get_targets());
     }
@@ -490,6 +495,7 @@ sub test {
     $this->_set_gopath();
     $this->_set_gocache();
     $this->_set_go111module();
+    $this->_set_goproxy();
     unshift @_, ('-p', $this->get_parallel());
     # Go 1.10 started calling “go vet” when running “go test”. This breaks tests
     # of many not-yet-fixed upstream packages, so we disable it for the time
